@@ -9,7 +9,10 @@ import pytest
 
 import torf
 
-from . import *  # noqa: F403
+from . import (ComparableException, calc_corruptions, calc_good_pieces,
+               calc_piece_indexes, calc_pieces_done, change_file_size,
+               display_filespecs, file_piece_indexes, file_range, fuzzylist,
+               pos2file, pos2files, random_positions, round_down_to_multiple)
 
 import logging  # isort:skip
 debug = logging.getLogger('test').debug
@@ -37,12 +40,15 @@ class CollectingCallback():
             else:
                 assert piece_hash is None
             self.seen_exceptions.append(ComparableException(exc))  # noqa: F405
+            debug(f'--- Seeing exception: {self.seen_exceptions[-1]}')
         elif piece_hash is None:
             assert exc is None
             self._seen_skipped_pieces[path.name].append(piece_index)
+            debug(f'--- Seeing skipped piece of {path.name}: {piece_index}')
         else:
             assert exc is None
             assert type(piece_hash) is bytes and len(piece_hash) == 20
+            debug(f'--- Seeing good piece of {path.name}: {piece_index}')
             self._seen_good_pieces[path.name].append(piece_index)
 
     @property
@@ -83,18 +89,23 @@ class _TestCaseBase():
     def run(self, *_, with_callback, exp_return_value=None, skip_on_error=False):
         debug(f'Original stream: {self.stream_original.hex()}')
         debug(f' Corrupt stream: {self.stream_corrupt.hex()}')
-        debug(f'Corruption positions: {self.corruption_positions}')
-        debug(f'Corrupt piece indexes: {set(corrpos // self.piece_size for corrpos in self.corruption_positions)}')
+        debug(f'Corruption positions: {sorted(self.corruption_positions)}')
+        debug(f'Corrupt piece indexes: {sorted(set(corrpos // self.piece_size for corrpos in self.corruption_positions))}')
 
         self.skip_on_error = skip_on_error
-        kwargs = {'skip_on_error': skip_on_error,
-                  'exp_return_value': exp_return_value}
+        kwargs = {
+            # 'skip_on_error': skip_on_error,
+            'exp_return_value': exp_return_value,
+        }
         if not with_callback:
             exp_exceptions = self.exp_exceptions
             if not exp_exceptions:
+                debug('Expecting no exceptions')
                 self._run_without_callback(**kwargs)
             else:
+                debug(f'Expected exceptions: {exp_exceptions}')
                 exp_exception_types = tuple(set(type(exc) for exc in exp_exceptions))
+                debug(f'Expected exception types: {exp_exception_types}')
                 with pytest.raises(exp_exception_types) as e:
                     self._run_without_callback(**kwargs)
                 # Usually the first error in the stream is reported, but not
@@ -117,19 +128,23 @@ class _TestCaseBase():
         kwargs['callback'] = cb
         kwargs['interval'] = 0
         if exp_return_value is not None:
-            assert self.torrent.verify(self.content_path, **kwargs) is exp_return_value
+            return_value = self.torrent.verify(self.content_path, **kwargs)
+            assert return_value == exp_return_value
         else:
             self.torrent.verify(self.content_path, **kwargs)
+
+        debug(f'seen_exceptions: {cb.seen_exceptions}')
+        assert cb.seen_exceptions == self.exp_exceptions
+
+        debug(f'seen_piece_indexes: {cb.seen_piece_indexes}')
+        assert cb.seen_piece_indexes == self.exp_piece_indexes
+
+        debug(f'seen_pieces_done: {cb.seen_pieces_done}')
+        assert cb.seen_pieces_done == self.exp_pieces_done
         # Last pieces_done value must be the total number of pieces so progress
         # is finalized correctly, e.g. progress bar ends at 100%
         assert cb.seen_pieces_done[-1] == self.torrent.pieces
 
-        debug(f'seen_exceptions: {cb.seen_exceptions}')
-        assert cb.seen_exceptions == self.exp_exceptions
-        debug(f'seen_piece_indexes: {cb.seen_piece_indexes}')
-        assert cb.seen_piece_indexes == self.exp_piece_indexes
-        debug(f'seen_pieces_done: {cb.seen_pieces_done}')
-        assert cb.seen_pieces_done == self.exp_pieces_done
         debug(f'seen_good_pieces: {cb.seen_good_pieces}')
         assert cb.seen_good_pieces == self.exp_good_pieces
 
@@ -147,7 +162,7 @@ class _TestCaseBase():
             self._exp_piece_indexes = calc_piece_indexes(self.filespecs, self.piece_size,  # noqa: F405
                                                          self.files_missing, self.files_missized)
             debug(f'Expected piece indexes: {dict(self._exp_piece_indexes)}')
-        return dict(self._exp_piece_indexes)
+        return self._exp_piece_indexes
 
     @property
     def exp_good_pieces(self):
@@ -157,11 +172,14 @@ class _TestCaseBase():
                                                      self.files_missing,
                                                      self.corruption_positions,
                                                      self.files_missized)
-            if self.skip_on_error:
-                self._exp_good_pieces = skip_good_pieces(self._exp_good_pieces,  # noqa: F405
-                                                         self.filespecs,
-                                                         self.piece_size,
-                                                         self.corruption_positions)
+            # This is disabled because the skip_on_error option for
+            # Torrent.verify() was removed. Feel free to re-implement and
+            # re-enable.
+            # if self.skip_on_error:
+            #     self._exp_good_pieces = skip_good_pieces(self._exp_good_pieces,  # noqa: F405
+            #                                              self.filespecs,
+            #                                              self.piece_size,
+            #                                              self.corruption_positions)
             debug(f'Expected good pieces: {self._exp_good_pieces}')
         return self._exp_good_pieces
 
@@ -171,11 +189,16 @@ class _TestCaseBase():
             self._exp_exc_corruptions = calc_corruptions(self.filespecs_abspath,  # noqa: F405
                                                          self.piece_size,
                                                          self.corruption_positions)
-            if self.skip_on_error:
-                self._exp_exc_corruptions = skip_corruptions(self._exp_exc_corruptions, self.filespecs_abspath,  # noqa: F405
-                                                             self.piece_size, self.corruption_positions,
-                                                             self.files_missing, self.files_missized)
-            debug(f'Expected corruptions: {self._exp_exc_corruptions}')
+            # This is disabled because the skip_on_error option for
+            # Torrent.verify() was removed. Feel free to re-implement and
+            # re-enable.
+            # if self.skip_on_error:
+            #     self._exp_exc_corruptions = skip_corruptions(self._exp_exc_corruptions, self.filespecs_abspath,  # noqa: F405
+            #                                                  self.piece_size, self.corruption_positions,
+            #                                                  self.files_missing, self.files_missized)
+            debug('Expected corruptions:')
+            for exc in self._exp_exc_corruptions:
+                debug(f'  {exc}')
         return self._exp_exc_corruptions
 
     @property
@@ -215,21 +238,24 @@ class _TestCaseBase():
 
             # If there are no missing or missized files, corruptions are mandatory
             if not mandatory:
-                debug('  all corruption exceptions are mandatory')
+                debug('all corruption exceptions are mandatory')
                 mandatory.update(self.exp_exc_corruptions)
                 maybe.update(self.exp_exc_corruptions.maybe)
             else:
-                # Corruptions must be reported if they don't exist in missing or
-                # missized files.
+                debug('not all corruption exceptions are mandatory')
+                # Corrupt files are only reported if their piece_indexes aren't
+                # already covered by missing or missized files
+                missing_missized_pis = set()
+                for filepath in itertools.chain(self.files_missing, self.files_missized):
+                    filename = os.path.basename(filepath)
+                    file_pis = file_piece_indexes(filename, self.filespecs, self.piece_size, exclusive=False)  # noqa: F405
+                    missing_missized_pis.update(file_pis)
                 for exc in self.exp_exc_corruptions:
-                    if any(filepath in itertools.chain(self.files_missing, self.files_missized)
-                           for filepath in exc.files):
+                    if exc.piece_index not in missing_missized_pis:
                         debug(f'  expecting non-missing/missized: {str(exc)}')
                         mandatory.add(exc)
-                    elif not all(filepath in itertools.chain(self.files_missing, self.files_missized)
-                                 for filepath in exc.files):
-                        debug(f'  expecting side-effect: {str(exc)}')
-                        mandatory.add(exc)
+                    else:
+                        debug(f'  not expecting missing/missized: {str(exc)}')
 
                 # Also allow corruptions that are already classified as optional.
                 for exc in self.exp_exc_corruptions.maybe:
@@ -243,6 +269,7 @@ class _TestCaseBase():
             debug('Tolerated exceptions:')
             for e in self._exp_exceptions.maybe:
                 debug(repr(e))
+
         return self._exp_exceptions
 
 class _TestCaseSinglefile(_TestCaseBase):
@@ -270,7 +297,7 @@ class _TestCaseSinglefile(_TestCaseBase):
         # Introduce random number of corruptions without changing stream length
         corruption_positions = set(random_positions(self.stream_corrupt) if not positions else positions)  # noqa: F405
         for corrpos in corruption_positions:
-            debug(f'Introducing corruption at index {corrpos}')
+            debug(f'* Introducing corruption at index {corrpos}')
             self.stream_corrupt[corrpos] = (self.stream_corrupt[corrpos] + 1) % 256
             self.content_path.write_bytes(self.stream_corrupt)
         self.corruption_positions.update(corruption_positions)
@@ -279,7 +306,7 @@ class _TestCaseSinglefile(_TestCaseBase):
         # Check if this file already has other errors
         if self.corruption_positions or self.files_missized:
             return
-        debug(f'Removing file from file system: {os.path.basename(self.content_path)}')
+        debug(f'* Removing file from file system: {os.path.basename(self.content_path)}')
         os.rename(self.content_path, str(self.content_path) + '.deleted')
         self.files_missing = [self.content_path]
         self.stream_corrupt = b'\xCC' * self.torrent.size
@@ -291,9 +318,10 @@ class _TestCaseSinglefile(_TestCaseBase):
         # Check if this file already has other errors
         if self.corruption_positions or self.files_missing:
             return
-        debug(f'Changing file size in file system: {os.path.basename(self.content_path)}')
+        debug(f'* Changing file size in file system: {os.path.basename(self.content_path)}')
         self.stream_corrupt = change_file_size(self.content_path, self.torrent.size)  # noqa: F405
         self.files_missized.append(self.content_path)
+        debug(f'  Corruption positions after changing file size: {self.corruption_positions}')
 
     def get_original_filesize(self, filepath):
         return len(self.stream_original)
@@ -345,14 +373,14 @@ class _TestCaseMultifile(_TestCaseBase):
             if filename in error_files:
                 continue
             else:
-                debug(f'Introducing corruption in {filename} at index {corrpos_in_stream} in stream, '
+                debug(f'* Introducing corruption in {filename} at index {corrpos_in_stream} in stream, '
                       f'{corrpos_in_file} in file {filename}')
                 self.corruption_positions.add(corrpos_in_stream)
                 data = self.content_corrupt[filename]
                 data[corrpos_in_file] = (data[corrpos_in_file] + 1) % 256
                 (self.content_path / filename).write_bytes(data)
                 self.files_corrupt.append(str(self.content_path / filename))
-        debug(f'Corruption positions after corrupting stream: {self.corruption_positions}')
+        debug(f'  Corruption positions after corrupting stream: {self.corruption_positions}')
 
     def delete_file(self, index=None):
         if index is None:
@@ -366,7 +394,7 @@ class _TestCaseMultifile(_TestCaseBase):
         if filename in error_files:
             return
 
-        debug(f'Removing file from file system: {os.path.basename(filename)}')
+        debug(f'* Removing file from file system: {os.path.basename(filename)}')
         filepath = self.content_path / filename
         os.rename(filepath, str(filepath) + '.deleted')
         self.files_missing.append(filepath)
@@ -389,7 +417,7 @@ class _TestCaseMultifile(_TestCaseBase):
 
         self.corruption_positions.update(corruption_positions)
         self._remove_skipped_corruptions()
-        debug(f'Corruption positions after removing file: {self.corruption_positions}')
+        debug(f'  Corruption positions after removing file: {self.corruption_positions}')
 
     def _remove_skipped_corruptions(self):
         # Finally, remove corruptions that exclusively belong to
@@ -409,7 +437,6 @@ class _TestCaseMultifile(_TestCaseBase):
         else:
             filename = tuple(self.content_original)[index]
         filepath = self.content_path / filename
-        debug(f'Changing file size in file system: {filepath}')
 
         # Don't change corrupt/missing file
         error_files = set(os.path.basename(f) for f in itertools.chain(
@@ -417,6 +444,8 @@ class _TestCaseMultifile(_TestCaseBase):
         if filename in error_files:
             return
 
+        debug(f'* Changing file size in file system: {filepath}')
+
         # Change file size
         self.content_corrupt[filename] = change_file_size(  # noqa: F405
             filepath, len(self.content_original[filename]))
@@ -439,7 +468,7 @@ class _TestCaseMultifile(_TestCaseBase):
                 self.corruption_positions.add(file_end)
 
         self._remove_skipped_corruptions()
-        debug(f'Corruption positions after changing file size: {self.corruption_positions}')
+        debug(f'  Corruption positions after changing file size: {self.corruption_positions}')
 
     def get_original_filesize(self, filepath):
         return len(self.content_original[os.path.basename(filepath)])
@@ -471,6 +500,63 @@ def test_validate_is_called_first(monkeypatch):
     assert str(excinfo.value) == 'Invalid metainfo: Mock error'
     mock_validate.assert_called_once_with()
 
+def test_verify_singlefile_torrent_with_directory(generated_singlefile_torrent, create_dir):
+    content_path = create_dir('multifile')
+    exp_exception = torf.VerifyIsDirectoryError(content_path)
+
+    # Without callback
+    with pytest.raises(type(exp_exception)) as excinfo:
+        generated_singlefile_torrent.verify(content_path)
+    assert str(excinfo.value) == str(exp_exception)
+
+    # With callback
+    cb = mock.Mock()
+    generated_singlefile_torrent.verify(content_path, callback=cb)
+    exp_torrent = generated_singlefile_torrent
+    exp_filepath = content_path
+    exp_pieces_done = 0
+    exp_pieces_total = generated_singlefile_torrent.pieces
+    exp_piece_index = 0
+    exp_piece_hash = None
+    assert cb.call_args_list == [mock.call(
+        exp_torrent,
+        exp_filepath,
+        exp_pieces_done,
+        exp_pieces_total,
+        exp_piece_index,
+        exp_piece_hash,
+        ComparableException(exp_exception),
+    )]
+
+def test_verify_multifile_torrent_with_file(generated_multifile_torrent, tmp_path):
+    content_path = tmp_path / 'singlefile'
+    content_path.write_text('some file data')
+    exp_exception = torf.VerifyNotDirectoryError(content_path)
+
+    # Without callback
+    with pytest.raises(type(exp_exception)) as excinfo:
+        generated_multifile_torrent.verify(content_path)
+    assert str(excinfo.value) == str(exp_exception)
+
+    # With callback
+    cb = mock.Mock()
+    generated_multifile_torrent.verify(content_path, callback=cb)
+    exp_torrent = generated_multifile_torrent
+    exp_filepath = content_path
+    exp_pieces_done = 0
+    exp_pieces_total = generated_multifile_torrent.pieces
+    exp_piece_index = 0
+    exp_piece_hash = None
+    assert cb.call_args_list == [mock.call(
+        exp_torrent,
+        exp_filepath,
+        exp_pieces_done,
+        exp_pieces_total,
+        exp_piece_index,
+        exp_piece_hash,
+        ComparableException(exp_exception),
+    )]
+
 def test_verify_content_successfully(mktestcase, piece_size, callback, filespecs):
     display_filespecs(filespecs, piece_size)  # noqa: F405
     tc = mktestcase(filespecs, piece_size)
@@ -484,13 +570,13 @@ def test_verify_content_with_random_corruptions_and_no_skipping(mktestcase, piec
     tc.run(with_callback=callback['enabled'],
            exp_return_value=False)
 
-def test_verify_content_with_random_corruptions_and_skipping(mktestcase, piece_size, callback, filespecs):
-    display_filespecs(filespecs, piece_size)  # noqa: F405
-    tc = mktestcase(filespecs, piece_size)
-    tc.corrupt_stream()
-    tc.run(with_callback=callback['enabled'],
-           skip_on_error=True,
-           exp_return_value=False)
+# def test_verify_content_with_random_corruptions_and_skipping(mktestcase, piece_size, callback, filespecs):
+#     display_filespecs(filespecs, piece_size)  # noqa: F405
+#     tc = mktestcase(filespecs, piece_size)
+#     tc.corrupt_stream()
+#     tc.run(with_callback=callback['enabled'],
+#            skip_on_error=True,
+#            exp_return_value=False)
 
 def test_verify_content_with_missing_files_and_no_skipping(mktestcase, piece_size, callback, filespecs, filespec_indexes):
     display_filespecs(filespecs, piece_size)  # noqa: F405
@@ -500,31 +586,29 @@ def test_verify_content_with_missing_files_and_no_skipping(mktestcase, piece_siz
     tc.run(with_callback=callback['enabled'],
            exp_return_value=False)
 
-def test_verify_content_with_missing_files_and_skipping(mktestcase, piece_size, callback, filespecs, filespec_indexes):
-    display_filespecs(filespecs, piece_size)  # noqa: F405
-    tc = mktestcase(filespecs, piece_size)
-    for index in filespec_indexes:
-        tc.delete_file(index)
-    tc.run(with_callback=callback['enabled'],
-           skip_on_error=True,
-           exp_return_value=False)
+# def test_verify_content_with_missing_files_and_skipping(mktestcase, piece_size, callback, filespecs, filespec_indexes):
+#     display_filespecs(filespecs, piece_size)  # noqa: F405
+#     tc = mktestcase(filespecs, piece_size)
+#     for index in filespec_indexes:
+#         tc.delete_file(index)
+#     tc.run(with_callback=callback['enabled'],
+#            skip_on_error=True,
+#            exp_return_value=False)
 
 def test_verify_content_with_changed_file_size_and_no_skipping(mktestcase, piece_size, callback, filespecs):
     display_filespecs(filespecs, piece_size)  # noqa: F405
-
     tc = mktestcase(filespecs, piece_size)
     tc.change_file_size()
     tc.run(with_callback=callback['enabled'],
            exp_return_value=False)
 
-def test_verify_content_with_changed_file_size_and_skipping(mktestcase, piece_size, callback, filespecs):
-    display_filespecs(filespecs, piece_size)  # noqa: F405
-
-    tc = mktestcase(filespecs, piece_size)
-    tc.change_file_size()
-    tc.run(with_callback=callback['enabled'],
-           skip_on_error=True,
-           exp_return_value=False)
+# def test_verify_content_with_changed_file_size_and_skipping(mktestcase, piece_size, callback, filespecs):
+#     display_filespecs(filespecs, piece_size)  # noqa: F405
+#     tc = mktestcase(filespecs, piece_size)
+#     tc.change_file_size()
+#     tc.run(with_callback=callback['enabled'],
+#            skip_on_error=True,
+#            exp_return_value=False)
 
 def test_verify_content_with_multiple_error_types(mktestcase, piece_size, callback, filespecs):
     display_filespecs(filespecs, piece_size)  # noqa: F405
@@ -535,5 +619,5 @@ def test_verify_content_with_multiple_error_types(mktestcase, piece_size, callba
         errorizer = errorizers.pop(random.choice(range(len(errorizers))))
         errorizer()
     tc.run(with_callback=callback['enabled'],
-           skip_on_error=random.choice((True, False)),
+           # skip_on_error=random.choice((True, False)),
            exp_return_value=False)
